#!/bin/bash
#$ -l h_rt=4:00:00
#$ -l rmem=5G
#$ -l mem=8G
#$ -t 1-500
#$ -pe openmp 2
#$ -V

# this is a trick to avoid running more 
# than one process per node setting the
# minimum scratch space per core required
# for Iceberg worker nodes, this must be 
# close or equal to 760GB (but not higher) 
# divided by the number of processors
# (i.e. 190GBx4cores=760GB)
# This effectively restricts jobs to
# the newer Iceberg nodes that have
# more scratch space (and are faster)
#
# --------------------------------------
#$ -l scratch=380g
# --------------------------------------

# number of processors per job
NPROC=2

# memory for iprlookup server
# (minimum 8GB)
ILMEM=8

# memory for actual interproscan
IMEM=2

# minimum nucleotide sze of ORF to report
MINSIZE=100

# In this example, you are supposed to use a
# directory called "interproscan" in your
# /fastdata directory. The protein sequences
# are expected to be in a subdirectory called
# "sequences" in the "interproscan" folder

# temporary directory
TMPDIR="/scratch/$USER/interproscan/tmp"
# TMPDIR="/fastdata/$USER/interproscan/tmp"
mkdir -p $TMPDIR >& /dev/null

# directory with sequences
INDIR="/fastdata/$USER/interproscan/sequences"

# output directory
OUTDIR="/fastdata/$USER/interproscan/res"
mkdir -p $OUTDIR >& /dev/null

# interproscan path
INTERPROSCAN='/usr/local/extras/Genomics/apps/interproscan/5.20-59.0/interproscan'

# Input files (protein fasta files)
INFILES=($(find $INDIR/ -name "*.fa"| sort -V))
INFILE=${INFILES[$SGE_TASK_ID-1]}
echo "INFILE $INFILE ($SGE_TASK_ID)"
hostname
date
echo "---------------------------------------------------------------------"
echo $INFILE
echo "---------------------------------------------------------------------"
echo "$INTERPROSCAN -nt $NPROC -ilmem $ILMEM -imem $IMEM -ms 100 -goterms -pa -iprlookup -i $INFILE -t p -T $TMPDIR -d $OUTDIR -f tsv,gff3,html,xml"
echo "---------------------------------------------------------------------"

$INTERPROSCAN -nt $NPROC -ilmem $ILMEM -imem $IMEM -ms 100 -goterms -pa -iprlookup -i $INFILE -t p -T $TMPDIR -d $OUTDIR -f raw,tsv,gff3,html,xml

echo "---------------------------------------------------------------------"
date
