#!/usr/bin/env perl
# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com

# Description:
# Reorientate annotations for scaffolds 702.1 and 2963 in a gff3 file
# to convert T. cristinae draft 1.3b2 to 1.3c2

use warnings;
use strict;

my $gff="timema_cristinae_1.3b2_braker_interproscan_predgene_and_funcann.gff3";
my $gffout="| bedtools sort -i - -faidx tcristinae_draft_1.3c2.fasta.fai > timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.gff3";
# scaffolds inverted and their length
my %scaf2invert=(
"lg8_ord14_scaf702.1"=>14171514,
"lg8_ord13_scaf2963"=>17133644);

my @newscaf=();
open (FILE, "$gff");
open (FILEOUT, "$gffout");
	while (<FILE>){
		chomp;
		my @aux=split(/\t/,$_);
		if ($scaf2invert{$aux[0]}){
			my $start=$scaf2invert{$aux[0]}-$aux[4]+1;
			my $stop=$scaf2invert{$aux[0]}-$aux[3]+1;
			my $orientation;
			if ($aux[6] eq '+'){
				$orientation='-';
			}
			else{
				$orientation='+';
			}
			# print "PREVIOUS: $_\n";
			print FILEOUT join("\t",@aux[0..2])."\t$start\t$stop\t$aux[5]\t$orientation\t".join("\t",@aux[7..8])."\n";
		}
		else{
			print FILEOUT "$_\n";
		}
	}

close(FILEOUT);
close(FILE);

system("echo '##gff-version 3' > timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3;
        echo '##timema_cristinae_1.3c2' >> timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3;
		echo '##annotations generated with braker 1.9 (using GeneMark-ET 4.32, AUGUSTUS 3.2.2) and InterProScan 5.20-59.0' >> timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3;
		echo '##using Iowa.ALL454Reads.fq.bz2 (Comeault et al. 2012 MolEcolRes doi: 10.1111/j.1755-0998.2012.03121.x) RNA reads aligned with STAR 2.5.1b' >> timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3;
		cat timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.gff3 >> timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3;
	    echo '##FASTA' >> timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3;
	    cat tcristinae_draft_1.3c2.fasta >> timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3;
		lbzip2 timema_cristinae_1.3c2_braker_interproscan_predgene_and_funcann.wseq.gff3");
