#!/bin/bash

# Mask genome with library of repeats

REPEATMASKER='/usr/local/extras/Genomics/apps/RepeatMasker/4.0.6/RepeatMasker'
REPEATLIB='RepeatLibMergeCentroidsRM.lib'
GENOME='tcristinae_draft_1.3b2.fasta'
NCORES=24
WORKDIR='/local/tmp/repeatmasker'
OUTDIR='./repeatmasker'

rm -rf $WORKDIR >& /dev/null
mkdir -p $WORKDIR
mkdir -p $OUTDIR

ln -s $REPEATLIB $WORKDIR/
ln -s $GENOME $WORKDIR/

REPEATLIB=$(basename $REPEATLIB)
GENOME=$(basename $GENOME)

cd $WORKDIR

date 
echo "====================================================================================="
echo "====================================================================================="
echo "====================================================================================="
echo

echo "Executing: "
echo "$REPEATMASKER \\"
echo "-s \\"
echo "-e ncbi \\"
echo "-pa $NCORES \\"
echo "-lib $REPEATLIB \\"
echo "-dir $OUTDIR \\"
echo "-xsmall \\"
echo "-gff \\"
echo "$GENOME"
echo

$REPEATMASKER \
-s \
-e ncbi \
-pa $NCORES \
-lib $REPEATLIB \
-dir $OUTDIR \
-xsmall \
-gff \
$GENOME

# convert to gff3
/usr/local/extras/Genomics/apps/repeatmasker/current/util/rmOutToGFF3.pl \
$OUTDIR/$GENOME.out \
> $OUTDIR/$GENOME.out.gff3

rsync -avPh $WORKDIR/* $OUTDIR/ 

rm -r $WORKDIR

echo "====================================================================================="
echo "====================================================================================="
echo "====================================================================================="
date
echo

