#!/bin/bash

# Align 454 RNA reads to masked genome

GENOME='tcristinae_draft_1.3b2.fasta.masked'
RNASEQ='Iowa.ALL454Reads.fq.bz2'
NCORES=24

OUTDIR='./star'
OUTPREFIX="$OUTDIR/out"
mkdir -p $OUTDIR/genome

date
echo "====================================================================================="
echo "====================================================================================="
echo "====================================================================================="
echo

echo "Executing: "
echo "STARlong \\"
echo "--runThreadN $NCORES \\"
echo "--runMode genomeGenerate \\"
echo "--genomeDir $OUTDIR/genome \\"
echo "--genomeFastaFiles $GENOME"
echo

# generate genome indexes
STARlong \
--runThreadN $NCORES \
--runMode genomeGenerate \
--genomeDir $OUTDIR/genome \
--genomeFastaFiles $GENOME

echo
echo "====================================================================================="
echo "====================================================================================="
echo

echo "STARlong \\"
echo "--runThreadN $NCORES \\"
echo "--genomeDir $OUTDIR/genome \\"
echo "--readFilesIn $RNASEQ \\"
echo "--readFilesCommand "lbzip2 -n $NCORES -dck" \\"
echo "--twopassMode Basic --twopass1readsN -1 \\"
echo "--outFilterMismatchNmax 999 --outFilterMismatchNoverLmax 0.05 \\"
echo "--outFileNamePrefix $OUTPREFIX \\"
echo "--outSAMtype BAM SortedByCoordinate"

echo

# align RNAseq reads
STARlong \
--runThreadN $NCORES \
--genomeDir $OUTDIR/genome \
--readFilesIn $RNASEQ \
--readFilesCommand "lbzip2 -n $NCORES -dck" \
--twopassMode Basic --twopass1readsN -1 \
--outFilterMismatchNmax 999 --outFilterMismatchNoverLmax 0.05 \
--outFileNamePrefix $OUTPREFIX \
--outSAMtype BAM SortedByCoordinate

echo "====================================================================================="
echo "====================================================================================="
echo "====================================================================================="
date
echo
