#! /usr/bin/env perl

# (c) Stuart Dennis modified by romain villoutreix
# stuart.dennis@sheffield.ac.uk / r.villoutreix@sheffield.ac.uk
# Last modified: 11/05/2016

# Description:
# This script creates initial input files for GenABEL analyses in R
# Requirements:  
# 1. a genotype file (.geno or .geno.csv) with a header

use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);


my $date=strftime("%Y%m%d%-H%M%S", localtime);

my $version='1.0-2016.09.08';
my $ncpu=1;



&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile, 
    'h|help' => \&usage 
);

&usage if (!defined($infile));


#Input file
if (! -e $infile){
die ("\nCan't find input file: $infile\n\n");
}
$infile=File::Spec->rel2abs($infile);


# Output files

my $outbase=basename($infile);
$outbase=~ s/\.geno//g;
$outbase=~ s/\.bbgeno//g;

my $out=basename($infile);
$out=~ s/\.geno/\.txt/g;
$out=~ s/\.bbgeno/\.txt/g;

print "Configuring input file $infile for conversion...\n\n";

##Add the line necessry for GenABEL. ids, SNP names, LG, position here filled as SNPs position on scaffold, and genotype 0: missing, 1: homozygote 1, 2: heterozygote, 3: homozygote 2 
system(qq(cut -d' ' -f4- $infile | awk '{if (NR==1) {print}}' > $out));
system(qq(awk '{if (NR!=1) {print \$1}}' $infile | awk '{printf "%s ",\$0} END {print ""}' >> $out));
system(qq(awk '{if (NR!=1) {print \$1}}' $infile  | awk -F: '{print \$1}' | awk -F_ '{print\$1}' | perl -pi -e 's/lg//g'| perl -pi -e 's/NA/14/g'| awk '{printf "%s ",\$0} END {print ""}' >> $out));
system(qq(awk '{if (NR!=1) {print \$1}}' $infile  | awk -F: '{print \$2}' | awk '{printf "%s ",\$0} END {print ""}' >> $out));

## round genotypes probabilities. 0: missing, 1: homozygote 1, 2: heterozygote, 3: homozygote 2; Round as follow : ]0;0.5[ : 1 / [0.5;1.5] : 2 / ]1.5;2] : 3
system(qq(cut -d' ' -f4- $infile | awk '{if (NR!=1) {print}}'  | perl -pe 's/[-+]?\\d*(?:\\.?\\d|\\d\\.)\\d*(?:[eE][-+]?\\d+)?/sprintf("%.0f",\$&)+1/ge' >> $out));

print "Done.\n\n Converting to GenABEL input format (this will take a few minutes).\n\n";


my $R = Statistics::R->new();

$R->run(qq`
		library(GenABEL)
			convert.snp.text("$out", paste("$outbase","genotype.raw",sep="_"), bcast=10000)`);
$R->stop();

# --------------------------------------------------------------------------------------------

unlink $out;
print "Conversion complete.  GenABEL input file saved as $outbase\_genotype.raw\n\n";

system("rm $out");
print "intermediate files $out deleted\n\n";


# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Stuart Dennis modified by Romain Villoutreix\n";
    print "  stuart.dennis\@sheffield.ac.uk / r.villoutreix\@sheffield.ac.uk\n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (.geno file / needs to have ids in the first line)>\n";
	print "      -h <show this help>\n";
    print "\n";
	print "  Note: function rounds genotypes as follow: \n    \]0;0.5\[ : homozygote 1\n\    [0.5;1.5\] : heterozygote\n\    ]1.5;2\] : homozygote 2\n\n";
    exit;
}
# ==============================================================================
