#!/usr/bin/env perl

# (c) Romain Viloutreix
# romain.villoutreix@gmail.com

# Description: Given a Gemma sparse matrix file returns file with chr and PS position filled
# It also extract scaffold and scaffold order values, and add the 'physical position' of SNPs 

# TODO : put the defaults files somewhere accessible from iceberg.

use warnings;
use Getopt::Long;
use File::Basename;
use LWP::Simple;
use List::Util qw(sum min);
use Statistics::R;

my $version='2016-10-18';

&author;
my ($infile, $outfile);
my $cumlenfile="/stash/home/bo1revx/romain_scripts/dioscripts/gwas/useful_files/tcristinae_draft_1.2b.scalen.scaf-1.cumlen.dsv"; 

GetOptions(
    'i|I=s' => \$infile, # sparse matrix file output from gemma_summary.pl (8 columns: CHR	RS	PS	alpha	beta	gamma	abs.beta.g	dir.beta.g)
	'p|P=s' => \$cumlenfile, # scaf-1 cumulative length file created using compute_scaf-1_cummulative_length.pl function.
	'o|O=s' => \$outfile,
    'h|help'=> \&usage
) or print ("\nERROR: Option/s not recognized (see above)\n") and &usage;


&usage if (!defined($infile));
&usage if (!defined($outfile));

# -----------------------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------------------
#formatting file

print STDERR "\n\n";

print "Necessary files found :) \n\n\  processing sparse file ! \n\n";

#Opening of the input and output files
my @fields=();
my @subfield1=();
my @subfield2=();
my $lg=();
my $scaffold=();
my $scaforder=();
my $position=();


open (OFH, ">$outfile\.tmp") or die "\nCan't open $outfile\.tmp file\n\n";
print OFH "CHR\tRS\tPS\talpha\tbeta\tgamma\tabs.beta.g\tdir.beta.g\tscaffold\tscaforder\n";

open (IFH, $infile) or die "\nCan't open $infile file\n\n";
	while (<IFH>){
	#skip the first line
	next if $. == 1;
	chomp $_;
	@fields = split("\t",$_);
	@subfield1 = split(":", $fields[1]);
	@subfield2 = split("_", $subfield1[0]);
	$lg = substr $subfield2[0], 2;
	$scaffold = substr $subfield2[2], 4;
	$scaforder = substr $subfield2[1], 3;
	$position = $subfield1[1];
	
	print OFH "$lg\t$subfield1[0]:$position\t$position\t$fields[3]\t$fields[4]\t$fields[5]\t$fields[6]\t$fields[7]\t$scaffold\t$scaforder\n";


	}

#closing the files
close(IFH);
close(OFH);


# R code - add real physical position
# # --------------------------------------------------------------------------------------------

my $R = Statistics::R->new();

$R->run(qq`

#### load formatted file with scaffold length
data <- read.table("$outfile\.tmp", header=TRUE, sep="\t", na.string="NA");

scaflen <- read.table("$cumlenfile", header=TRUE, sep="\t", na.string="NA");

data\$genome_pos_w_Ns <- NA;

for(i in c(unique(data\$scaffold))){
data\$genome_pos_w_Ns[data\$scaffold==i] <- data\$PS[data\$scaffold==i] + scaflen\$cumlength_w_Ns[scaflen\$scafnumber==i];
}

write.table(data, file="$outfile", row.names=FALSE, col.names=TRUE, dec=".", na="NA", eol="\n", sep="\t", quote=FALSE);`);

$R->stop();

system("rm $outfile\.tmp");

print "Bye ! \n\n";


# ================================================================================================
# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version     \n";
    print "  (c) Romain Villoutreix             \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <sparse effects file from gemma_summary.pl>\n";
	print "      -p <scaffold -1 cummulative length file from compute_scaf-1_cummulative_length.pl function, default v1.2b of the genome>\n";
	print "      -o <output file>\n";
    print "\n";
    exit;
}
# ==============================================================================
