#!/usr/bin/env perl

# (c) Romain Villoutreix
# romain.villoutreix@gmail.com
# Last modified: 18/10/2016

# Description:
# This script plots the sparse values (PIPs along the genome) resulting from the gemma_sparse_formatting.pl function
# generate barplot for mean PIP value along LG


use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);

my $version='2.0.1-2016.10.25';

#Default graphic variables
my $y1=0;
my $y2=0;
my $cumlenfile='/stash/home/bo1revx/romain_scripts/dioscripts/gwas/useful_files/tcristinae_draft_1.3b2.scalen.scaf-1.cumlen.dsv';

#Default outfiles
my $outdir='output';
&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile, 
    'o|O=s'  => \$outdir,
	's|S=s' => \$cumlenfile,
	'y1=f' => \$y1,
	'y2=f' => \$y2,
    'h|help' => \&usage 
);

&usage if (!defined($infile) || !defined($outdir));

#Input file does exist ?
if (! -e $infile){
die ("\nCan't access input file: $infile\n\n");
}

# Output directory does exist ?
if (! -e $outdir){
eval {make_path($outdir)}
or die ("\nError creating output directory: $outdir\n\n");
}

#Cumlen file does exist ?
if (! -e $cumlenfile){
die ("\nCan't access scaf-1 cumlenfile : $cumlenfile\n\n");
}
	   


#some specifications of path
$infile=File::Spec->rel2abs($infile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($infile);
$inbase=~ s/\.txt//g;
my $outbase="$outdir/".basename($infile);
$outbase=~ s/\.txt//g;

print "Necessary files found :) \n\n\  Making graphics ! \n\n"; 

# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------
   my $R = Statistics::R->new();
     
$R->run(qq`

setwd("$outdir");

#defining graphic variables

	y1 <- $y1;
	y2 <- $y2;

# opening sparse file and cumlenfile

	sparse <- read.table(file="$infile", sep="\t", dec=".", header=TRUE);

	scaflen <- read.table("$cumlenfile", header=TRUE, sep="\t", na.string="NA");
	scaflen <- scaflen[scaflen\$lg!=14,];

#discard unmapped LG

	sparsemapped <- sparse[!is.na(sparse\$CHR),];

#Generate the histogram of mean PIP per LG 
	
	meangammaperLG <- aggregate(sparsemapped[, match("gamma", colnames(sparsemapped))], list(sparsemapped\$CHR), mean);
	
	pdf(file="$outbase\_mean_PIP_per_LG.pdf");

	if(y1==0){barplot(height=meangammaperLG\$x, names.arg=meangammaperLG\$Group.1, ylab = "Mean PIP", xlab="LG", col ="gray");}
	if(y1!=0){barplot(height=meangammaperLG\$x, names.arg=meangammaperLG\$Group.1, ylab = "Mean PIP", xlab="LG", ylim=c(0,y1), col ="gray");}

	dev.off();

# plot the PIP values along the LG

	sparsemapped\$colour <- "NA";
	sparsemapped\$colour[sparsemapped\$CHR%%2==0] <- "red";
	sparsemapped\$colour[sparsemapped\$CHR%%2!=0] <- "yellow2";

	###taking into account physical map ###
	# prepare useful table for plot

    indextable <- aggregate(sparsemapped[,match("genome_pos_w_Ns", colnames(sparsemapped))], list(sparsemapped\$CHR), mean);

	jpeg(filename="$outbase\_PIP_along_LG_w_phy_pos.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

    if(y2==0){
		
		 plot(NULL, NULL, xlab = "linkage group", ylab = "PIP", ylim = c(0, max(sparsemapped\$gamma, na.rm=TRUE)),  xlim = c(0,(max(scaflen\$cumlength_w_Ns)+tail(scaflen\$length_w_Ns, 1))), bty = "n", xaxt = "n");
		 points(sparsemapped\$genome_pos_w_Ns, sparsemapped\$gamma, col = c(sparsemapped\$colour), cex = 0.65, pch=16);
		 text(x=c(indextable[,2]), y = rep((0-(max(sparsemapped\$gamma, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.8);
		
	}

    if(y2!=0){
		
		plot(NULL, NULL, xlab = "linkage group", ylab = "PIP", ylim = c(0, y2),  xlim = c(0,(max(scaflen\$cumlength_w_Ns)+tail(scaflen\$length_w_Ns, 1))), bty = "n", xaxt = "n");
		points(sparsemapped\$genome_pos_w_Ns, sparsemapped\$gamma, col = c(sparsemapped\$colour), cex = 0.65, pch=16);
		text(x=c(indextable[,2]), y = rep((0-(y2-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.8);
		
	}


	dev.off();`);

$R->stop();

print " Done!  Graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (sparse file output from gemma_sparse_formatting.pl)>\n";
	print "      -o <output directory - optional defaults to output/>\n";
	print "      -s <scaffold -1 cummulative length file from compute_scaf-1_cummulative_length.pl function, default v1.3b2 of the genome>\n";
	print "      -y1 <max y display value for mean PIP per LG graphic / default = automatic value>\n";
	print "      -y2 <max y display value for PIP along LG graphic / default = automatic value>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
