#!/usr/bin/env perl

# (c) Romain Villoutreix
# romain.villoutreix@gmail.com
# Last modified: 21/09/2016

# Description:
# This script plots the sparse values (PIPs) for a given LG resulting from the gemma_sparse_formatting.pl function


use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);
use List::Util 1.33 'any';

my $version='2.0.0-2016.10.18';

#Default graphic variables
my $y=0;
my $phypos=0;

#default LG=8
my $interstLG=8;

#default scaf -1 cumulative length file
my $scaflenfile="/stash/home/bo1revx/romain_scripts/dioscripts/gwas/useful_files/tcristinae_draft_1.2b.scalen.scaf-1.cumlen.dsv"; 

#Default outfiles
my $outdir='output';
&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile,
    'l|L=i' => \$interstLG,
	'p|P=i' => \$phypos,
	's|S=s' => \$scaflenfile,
    'o|O=s'  => \$outdir,
	'y=f' => \$y,
    'h|help' => \&usage 
);

&usage if (!defined($infile) || !defined($outdir));

#Input file does exist ?
if (! -e $infile){
die ("\nCan't access input file: $infile\n\n");
}

# Output directory does exist ?
if (! -e $outdir){
eval {make_path($outdir)}
or die ("\nError creating output directory: $outdir\n\n");
}

#verification that LG given exist
my @LGnumber = (1,2,3,4,5,6,7,8,9,10,11,12,13); 
my $match_found = any { /$interstLG/ } @LGnumber;

if($match_found==0){
die ("\nLG specified not good: must be an integer between 1-14\n\n");
}

#Input file does exist ?
if (! -e $scaflenfile){
die ("\nCan't access scaf-1 cumulative scaffold length file: $scaflenfile\n\n");
}


#some specifications of path
$infile=File::Spec->rel2abs($infile);
$outdir=File::Spec->rel2abs($outdir);
$scaflenfile=File::Spec->rel2abs($scaflenfile);
my $inbase=basename($infile);
$inbase=~ s/\.txt//g;
my $outbase="$outdir/".basename($infile);
$outbase=~ s/\.txt//g;

print "Necessary files found :) \n\n\  Making graphics ! \n\n"; 

# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------
   my $R = Statistics::R->new();
     
$R->run(qq`

setwd("$outdir");

#defining graphic variables

	y <- $y;
	phypos <- $phypos;

# opening sparse file and scaf-1 cumulative length file

	sparse <- read.table(file="$infile", sep="\t", dec=".", header=TRUE);
	scaflenfile <- read.table(file="$scaflenfile", sep="\t", dec=".", header=TRUE);

#change with the other function ! define locus on no LG as associated with lg 14, and scaffold order 0 

	sparse\$CHR[is.na(sparse\$CHR)] <- 14;
	sparse\$scaforder[sparse\$CHR==14] <- 0;
	sparsemapped <- sparse;

#select the LG of interest

	sparsemapped <- sparsemapped[sparsemapped\$CHR==$interstLG,];

# plot the PIP values along the interest LG in indexed order

	sparsemappedordered <- sparsemapped[order(sparsemapped\$CHR, sparsemapped\$scaforder, sparsemapped\$scaffold, sparsemapped\$PS),];
	sparsemappedordered\$snpindex <- 1:nrow(sparsemappedordered);
	sparsemappedordered\$colour <- "NA";
	for (i in 1:length(unique(sparsemappedordered\$scaffold))) {
	if(i%%2==0){sparsemappedordered\$colour[sparsemappedordered\$scaffold==unique(sparsemappedordered\$scaffold)[i]] <- "red";}
	if(i%%2!=0){sparsemappedordered\$colour[sparsemappedordered\$scaffold==unique(sparsemappedordered\$scaffold)[i]] <- "yellow2";}
	}


#with physical positions
	if(phypos==0){
	# prepare useful table for plot
	indextable <- aggregate(sparsemappedordered[,match("genome_pos_w_Ns", colnames(sparsemappedordered))], list(sparsemappedordered\$scaffold), mean);
	jpeg(filename="$outbase\_PIP_along_LG$interstLG\_w_phypos.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

	if(y==0){
	plot(NULL, NULL, xlab = "linkage group $interstLG", ylab = "PIP", ylim = c(0, max(sparsemappedordered\$gamma, na.rm=TRUE)),  xlim = c(min(scaflenfile\$cumlength_w_Ns[scaflenfile\$lg==$interstLG]), min(scaflenfile\$cumlength_w_Ns[scaflenfile\$lg==($interstLG+1)])), bty = "n", xaxt = "n");
	points(sparsemappedordered\$genome_pos_w_Ns, sparsemappedordered\$gamma, col = c(sparsemappedordered\$colour), cex = 0.45, pch=16);
	text(x=c(indextable[,2]), y = rep((0-(max(sparsemappedordered\$gamma, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.45, srt=90);
	}
	
	if(y!=0){
	plot(NULL, NULL, xlab = "linkage group $interstLG", ylab = "PIP", ylim = c(0, y),  xlim = c(min(scaflenfile\$genome_pos_w_Ns[scaflenfile\$lg==$interstLG]), min(scaflenfile\$genome_pos_w_Ns[scaflenfile\$lg==($interstLG+1)])), bty = "n", xaxt = "n");
	points(sparsemappedordered\$genome_pos_w_Ns, sparsemappedordered\$gamma, col = c(sparsemappedordered\$colour), cex = 0.45, pch=16);
	text(x=c(indextable[,2]), y = rep((0-(y-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.45, srt=90);
	}
	}

#without physical positions
    if(phypos==1){
    # prepare useful table for plot
    indextable <- aggregate(sparsemappedordered[,match("snpindex", colnames(sparsemappedordered))], list(sparsemappedordered\$scaffold), mean);
    jpeg(filename="$outbase\_PIP_along_LG$interstLG\_wo_phypos.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

    if(y==0){
    plot(NULL, NULL, xlab = "linkage group $interstLG", ylab = "PIP", ylim = c(0, max(sparsemappedordered\$gamma, na.rm=TRUE)),  xlim = c(0,max(sparsemappedordered\$snpindex)), bty = "n", xaxt = "n");
    points(sparsemappedordered\$snpindex, sparsemappedordered\$gamma, col = c(sparsemappedordered\$colour), cex = 0.45, pch=16);
    text(x=c(indextable[,2]), y = rep((0-(max(sparsemappedordered\$gamma, na.rm=TRUE)-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.45, srt=90);
    }
     
    if(y!=0){
    plot(NULL, NULL, xlab = "linkage group $interstLG", ylab = "PIP", ylim = c(0, y),  xlim = c(0,max(sparsemappedordered\$snpindex)), bty = "n", xaxt = "n");
    points(sparsemappedordered\$snpindex, sparsemappedordered\$gamma, col = c(sparsemappedordered\$colour), cex = 0.45, pch=16);
    text(x=c(indextable[,2]), y = rep((0-(y-0)*0.10), nrow(indextable)), labels = c(indextable[,1]), xpd=TRUE, cex=0.45, srt=90);
    }
    }

	dev.off();`);

$R->stop();

print " Done!  Graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (sparse file output from gemma_sparse_formatting.pl)>\n";
	print "      -l <Linkage group of interest (designed for T. cristinae: 1-13; can't display LGNA at the moment, shall be implemented in the future) / default=$interstLG>\n";
	print "      -o <output directory - optional defaults to output/>\n";
	print "      -p <physical position ? (0:yes ; 1:no)  - optional - default $phypos>\n";
	print "      -s <path scaf-1 cumulative length file - optional - default v1.2b reference genome>\n";
	print "      -y <max y display value for PIP along LG of interest / default = automatic value>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
