#!/usr/bin/env perl

# (c) Romain Villoutreix
# romain.villoutreix@gmail.com
# Last modified: 08/11/2017

# Description:
# This script plots the sparse values (PIPs) for a given position on a scaffold generated by the gemma_sparse_formatting.pl function


use warnings;
use strict;
use Getopt::Long;
use File::Basename;
use File::Path qw(make_path);
use File::Spec;
use Statistics::R;
use POSIX qw(strftime);
use List::Util 1.33 'any';

my $version='1.0.0-2017.11.08';

#Default graphic variables
my $y=0;

#default scaffold 128, start 5000000 end 6000000
my $interstscaf=128;
my $start=5000000;
my $end=6000000;

#Default outfiles
my $outdir='output';
&author;
my ($infile);
GetOptions( 
    'i|I=s'  => \$infile,
    's|S=f' => \$interstscaf,
	'st=i' => \$start,
	'en=i' => \$end,
    'o|O=s'  => \$outdir,
	'y=f' => \$y,
    'h|help' => \&usage 
);

&usage if (!defined($infile) || !defined($outdir));

#Input file does exist ?
if (! -e $infile){
die ("\nCan't access input file: $infile\n\n");
}

# Output directory does exist ?
if (! -e $outdir){
eval {make_path($outdir)}
or die ("\nError creating output directory: $outdir\n\n");
}

#verification that given scaffold exist !! TO DO !!
#my @LGnumber = (1,2,3,4,5,6,7,8,9,10,11,12,13,14); 
#my $match_found = any { /$interstLG/ } @LGnumber;
#if($match_found==0){
#die ("\nLG specified not good: must be an integer between 1-14\n\n");
#}


#some specifications of path
$infile=File::Spec->rel2abs($infile);
$outdir=File::Spec->rel2abs($outdir);
my $inbase=basename($infile);
$inbase=~ s/\.txt//g;
my $outbase="$outdir/".basename($infile);
$outbase=~ s/\.txt//g;

print "Necessary files found :) \n\n\  input file used: $infile\n\n"; 

print "  Scaffold: $interstscaf - start: $start - end: $end\n\n";

print "  Making graphics !\n\n";

# R code - put whatever you want in here
# --------------------------------------------------------------------------------------------
   my $R = Statistics::R->new();
     
$R->run(qq`

setwd("$outdir");

#defining graphic variables

	y <- $y;


# opening sparse file

	sparse <- read.table(file="$infile", sep="\t", dec=".", header=TRUE);

#select the scaffold of interest

	sparsemapped <- sparse[sparse\$scaffold==$interstscaf,];
	sparsemapped <- sparsemapped[((sparsemapped\$PS>=$start) & (sparsemapped\$PS<=$end)), ];

# plot the PIP values along the interest scaffold

	sparsemapped\$colour <- "grey";

	jpeg(filename="$outbase\_PIP_along_scaf_$interstscaf\_$start\_$end.jpg", width=21, height=10, units="cm", pointsize=12, res=600);

	if(y==0){
	plot(NULL, NULL, xlab = "scaffold $interstscaf", ylab = "PIP", ylim = c(0, max(sparsemapped\$gamma, na.rm=TRUE)),  xlim = c($start, $end), bty = "n", xaxt = "n");
	points(sparsemapped\$PS, sparsemapped\$gamma, col = c(sparsemapped\$colour), cex = 0.95);
	axis(1, seq($start, $end, 50000));
	}
	
	if(y!=0){
	plot(NULL, NULL, xlab = "scaffold $interstscaf", ylab = "PIP", ylim = c(0, y),  xlim = c($start, $end), bty = "n", xaxt = "n");
	points(sparsemapped\$PS, sparsemapped\$gamma, col = c(sparsemapped\$colour), cex = 0.95);
	axis(1, seq($start, $end, 50000));
	}

	dev.off();`);

$R->stop();

print " Done!  Graphs saved in $outdir \n\n  Bye \n\n";

# --------------------------------------------------------------------------------------------#


# ==============================================================================
# ==============================================================================
# ============================== SUBROUTINES ===================================
# ==============================================================================
# ==============================================================================

# Show copyright
# ==============================================================================
sub author{
    print "\n";
    print "#########################################\n";
    print "  ".basename($0)."\n";
	print "  version $version\n";
    print "  (c) Romain Villoutreix            \n";
    print "  romain.villoutreix\@gmail.com      \n";
    print "#########################################\n";
	print "\n";
}
# ==============================================================================

# Show usage
# ==============================================================================
sub usage{
    print "\n";
	print "  Usage:\n";
    print "    ".basename($0)."\n";
	print "      -i <input file (sparse file output from gemma_sparse_formatting.pl)>\n";
	print "      -s <scaffold of interest (designed for T. cristinae - no control of scaffold number given) / default=$interstscaf>\n";
	print "      -st <start on scaffold (designed for T. cristinae - no control of scaffold number given) / default=$start>\n";
	print "      -en <start on scaffold (designed for T. cristinae - no control of scaffold number given) / default=$end>\n";
	print "      -o <output directory - optional defaults to output/>\n";
	print "      -y <max y display value for PIP along LG of interest / default = automatic value>\n";
	print "      -h <show this help>\n";
    print "\n";
    exit;
}
# ==============================================================================
