#!/bin/bash

# (c) 2020 - Victor Soria-Carrasco
# victor.soria.carrasco@gmail.com
# Last modified: 20/07/2017 17:02:39

# Description:
# This script produces matrices with haplotypes (encoded as 0/1) using
# fastPHASE for phasing

# programs and scripts
# -----------------------------------------------------------------------------
BCFTOOLS="bcftools"
FASTPHASE="fastPHASE"
BCF2GL="/usr/local/extras/Genomics/scripts/tools/bcf2gl.pl"
GL2FASTPHASE="/usr/local/extras/Genomics/scripts/phasing/mkfastphaseinp.pl"
# -----------------------------------------------------------------------------

BCFFILE="Tbartmani_JL.allgreen.bin.bial.noindel.qs20.cov50.mdp204Mdp4080.maf0.01.bcf"
OUTDIR="./11_phasing"

COV="0.85" # sample coverage
LG="8" # linkage group
cd $OUTDIR
mkdir fastPHASE_res >& /dev/null
mkdir tmp >& /dev/null

BASENAME=$(basename $BCFFILE | perl -pe 's/\.bcf//g').cov$(perl -e 'print '$COV'*100').lg$LG

cd tmp

# filter out by coverage and restrict it to specific LG
NSAMPLES=$($BCFTOOLS query -l $BCFFILE | wc -l)
$BCFTOOLS view $BCFFILE -e '(AN/2)<('$NSAMPLES'*'$COV')' | \
grep -P "^#|^lg$LG" > $BASENAME.vcf

# convert to gl format
$BCF2GL \
-i $BASENAME.vcf \
-o $BASENAME.gl

# convert gl to fastPHASE input file
$GL2FASTPHASE $BASENAME.gl

cd ..

# Run fastPHASE
$FASTPHASE \
-ofastPHASE_res/$BASENAME \
-T20 -C35 \
-KL10 -KU20 -Ki2 \
-em4 \
-Pp \
-q0.9 \
tmp/$BASENAME.inp


# Use uncertain data as the rest
# chr1
echo -n "sample " > $BASENAME.chr1.dsv
tail -n+2 tmp/$BASENAME.snps | awk '{print $3"-"$4}' |  perl -pe 's/\n/ /g;' >> $BASENAME.chr1.dsv
echo >> $BASENAME.chr1.dsv
sed -n '/^15/,+1p' fastPHASE_res/$BASENAME"_hapguess_switch.out" | perl -pe 's/ |\[|\]//g' | perl -pe 'if (/^15/){s/\n/ /g}else{s/(?=.)(?<=.)/ /g}' >> $BASENAME.chr1.dsv

# chr2
echo -n "sample " > $BASENAME.chr2.dsv
tail -n+2 tmp/$BASENAME.snps | awk '{print $3"-"$4}' |  perl -pe 's/\n/ /g;' >> $BASENAME.chr2.dsv
echo >> $BASENAME.chr2.dsv
sed -n '/^15/{p;n;n;p}' fastPHASE_res/$BASENAME"_hapguess_switch.out" |  perl -pe 's/ |\[|\]//g' | perl -pe 'if (/^15/){s/\n/ /g}else{s/(?=.)(?<=.)/ /g}' >> $BASENAME.chr2.dsv


# Set uncertain data as gaps

# chr1
echo -n "sample " > $BASENAME.chr1gaps.dsv
tail -n+2 tmp/$BASENAME.snps | awk '{print $3"-"$4}' |  perl -pe 's/\n/ /g;' >> $BASENAME.chr1gaps.dsv
echo >> $BASENAME.chr1gaps.dsv
sed -n '/^15/,+1p' fastPHASE_res/$BASENAME"_hapguess_switch.out" | perl -pe 's/ //g; s/\[[0-1]\]/-/g' | perl -pe 'if (/^15/){s/\n/ /g}else{s/(?=.)(?<=.)/ /g}' >> $BASENAME.chr1gaps.dsv

# chr2
echo -n "sample " > $BASENAME.chr2gaps.dsv
tail -n+2 tmp/$BASENAME.snps | awk '{print $3"-"$4}' |  perl -pe 's/\n/ /g;' >> $BASENAME.chr2gaps.dsv
echo >> $BASENAME.chr2gaps.dsv
sed -n '/^15/{p;n;n;p}' fastPHASE_res/$BASENAME"_hapguess_switch.out" | perl -pe 's/ //g; s/\[[0-1]\]/-/g' | perl -pe 'if (/^15/){s/\n/ /g}else{s/(?=.)(?<=.)/ /g}' >> $BASENAME.chr2gaps.dsv


