#!/bin/bash

# Extract list of SNPs and, if need be, recalculate allele dosage to indicate melanic allele dosage (instead of minor allele dosage)

INPUT="Tchumash_GR8.06.latRG.bial.noindel.qs20.cov50.mdp265Mdp5310.maf0.01.lgNA.excluded.geno"
INDIV="Tchumash_GR8.06.latRG.bial.noindel.qs20.cov50.mdp265Mdp5310.maf0.01.samples"
OUTPUT="10snps_lg8scaf128_latRG.dsv"

# 10 SNPs from scaffold 128 (LG8) most strongly associated with latRG scores
SNPS=(
lg8_ord15_scaf128:5034274 lg8_ord15_scaf128:5034292 lg8_ord15_scaf128:5177826
lg8_ord15_scaf128:5177853 lg8_ord15_scaf128:5763047 lg8_ord15_scaf128:5765419
lg8_ord15_scaf128:5765424 lg8_ord15_scaf128:5765428 lg8_ord15_scaf128:5765432
lg8_ord15_scaf128:5796836
 )

# Indicate for which SNPs genotype probabilities have to be modified (1)
# because melanic allele is major allel (and original genotype probabilities
# indicate minor allele dosage)
REV=(
0 1 1 
1 0 0
1 1 1
1
)

# header for output file
echo -n "SNP green melanic " > $OUTPUT
cat $INDIV | perl -pe 's/\n/ /g;' | perl -pe 's/ $//g;$_.="\n";'>> $OUTPUT


# SNPs with genotype probabilities recalculated and alleles swapped if need
# be (so that first column is always "green allele" and second column is always
# "melanic allele")
for i in ${!SNPS[@]};
do
	echo $i ${SNPS[$i]}

	if [[ ${REV[$i]} == 1 ]];
	then
		grep "${SNPS[$i]}" $INPUT | \
		perl -pe '
		 chomp;
		 @aux=split(/ /,$_);
		 @aux2=($aux[0],$aux[2],$aux[1]);
		 foreach $i (3..$#aux){
			 push(@aux2,(2-$aux[$i]));
	     }
		 $_=join(" ",@aux2)."\n";' >> $OUTPUT
	else
		grep "${SNPS[$i]}" $INPUT >> $OUTPUT
	fi
done

